//=============================================================================
//
// FSSound module for MS Flight Simulator 98, 2000 and Combat Flight Simulator
// Copyright (c) 1999-2000 by Daniel Steiner <dstbon@bigfoot.com>
//
// This module allows you to use sound files in FS gauges trough an easy to use
// interface.
//
//=============================================================================
#include <windows.h>
#include <mmsystem.h>
#ifdef __GNUC__                     // set the correct path to the DirectX SDK
#include <directx/dsound.h>
#else
#include <dsound.h>
#endif

//-----------------------------------------------------------------------------
// general structure for handling the sound buffers
// note: let the FSSound.dll handle this structure, simply provide a
//       LPFSSOUND pointer when calling one of the interface routines
//-----------------------------------------------------------------------------
typedef struct _FSSOUND {
	DWORD code;                   // holds the coded file name of the wave file
	LPDIRECTSOUNDBUFFER lpDsb;    // pointer to a DirectSoundBuffer interface
	struct _FSSOUND *next;        // points to the next element in the list
} FSSOUND, *LPFSSOUND, **LPLPFSSOUND;


//-----------------------------------------------------------------------------
// prototypes of exported procedures
//-----------------------------------------------------------------------------
// create a sound buffer and register a wave file with it
BOOL RegisterFile(LPLPFSSOUND, PCHAR);

// create a sound buffer and register a wave resource with it
BOOL RegisterRes(LPLPFSSOUND, UINT, HMODULE);

// release the sound buffer
void Unregister(LPLPFSSOUND);

// play sound buffer, specify whether looping is required
BOOL Play(LPFSSOUND, BOOL);

// stop playing of a looping buffer
BOOL Stop(LPFSSOUND);

// checks whether a buffer is playing
BOOL IsPlaying(LPFSSOUND);

// returns the volume of a buffer
LONG GetVolume(LPFSSOUND);

// set the volume of a sound buffer
BOOL SetVolume(LPFSSOUND, LONG);

// returns the pan of a buffer
LONG GetPan(LPFSSOUND);

// set the pan of a sound buffer
BOOL SetPan(LPFSSOUND, LONG);

// returns the frequency of a buffer
DWORD GetFrequency(LPFSSOUND);

// set the frequency of a sound buffer
BOOL SetFrequency(LPFSSOUND, DWORD);


//-----------------------------------------------------------------------------
// some macros for convenience
//-----------------------------------------------------------------------------
// macros for handling sound files
#define MFSSoundFileVars(SOUND, NAME)										\
	struct {BOOL ok; LPFSSOUND buf; PCHAR name;} SOUND = {FALSE, NULL, NAME}

#define MFSSoundFileRegister(SOUND)											\
	SOUND.ok = RegisterFile(&(SOUND.buf), SOUND.name)

#define MFSSoundFilePlay(SOUND, LOOP)										\
	if (SOUND.ok && !Play(SOUND.buf, LOOP))									\
		if ((SOUND.ok = RegisterFile(&(SOUND.buf), SOUND.name)) == TRUE)	\
			Play(SOUND.buf, LOOP)											\

// macros for handling sound resources
#define MFSSoundResVars(SOUND, RES)											\
	struct {BOOL ok; LPFSSOUND buf; UINT res; HMODULE module;} SOUND		\
		= {FALSE, NULL, RES, NULL}

#define MFSSoundResRegister(SOUND, MODULE)									\
	SOUND.module=MODULE;													\
	SOUND.ok = RegisterRes(&(SOUND.buf), SOUND.res, MODULE)

#define MFSSoundResPlay(SOUND, LOOP)										\
	if (SOUND.ok && !Play(SOUND.buf, LOOP))									\
		if ((SOUND.ok = RegisterRes(&(SOUND.buf), SOUND.res, SOUND.module))	\
				== TRUE)													\
			Play(SOUND.buf, LOOP)											\

// common macros
#define MFSSoundUnregister(SOUND)											\
	Unregister(&(SOUND.buf))

#define MFSSoundStop(SOUND)													\
	Stop(SOUND.buf)

#define MFSSoundGetVolume(SOUND)											\
	GetVolume(SOUND.buf)

#define MFSSoundSetVolume(SOUND, VOL)										\
	(SOUND.ok && SetVolume(SOUND.buf, VOL))

#define MFSSoundIsPlaying(SOUND)											\
	(SOUND.ok && IsPlaying(SOUND.buf))

#define MFSSoundGetPan(SOUND)												\
	GetPan(SOUND.buf)

#define MFSSoundSetPan(SOUND, PAN)											\
	(SOUND.ok && SetPan(SOUND.buf, PAN))

#define MFSSoundGetFrequency(SOUND)											\
	GetFrequency(SOUND.buf)

#define MFSSoundSetFrequency(SOUND, FREQ)									\
	(SOUND.ok && SetFrequency(SOUND.buf, FREQ))
