// SDK.Magnetos.c
// Copyright (c) 1997 Microsoft Corporation.  All rights reserved.
//
// Copied from Sopwith Magnetos gauge
//
// Adapted to use the FSSound module by Daniel Steiner
//

// include the header file
#include	"FSSound.h"

#include	"gauges.h"
#include	"SDK.Magnetos.h"

//	Set up import and export tables and gauge header
char		gauge_name[]		= "SDK.Magnetos\0";
extern PELEMENT_HEADER		list;
MODULE_SETUP( GAUGE_HEADER_VERSION, 57, gauge_name, &list );

// set up the vars for two sound buffers
// the first one uses a sound file in the main FS sound folder
MFSSoundFileVars(sound1, "Sound\\Flaps.wav");
// the second one uses a sound file that is compiled into the gauge
MFSSoundResVars(sound2, SND_MARKER);


FAILURE_RECORD	fail[] =
{
	{FAIL_SYSTEM_ENGINE, FAIL_ACTION_NONE},
	{FAIL_NONE, FAIL_ACTION_NONE}
};


MODULE_VAR		starter	= {ENGINE1_STARTER_SWITCH_POS};

MAKE_ICON(		icon_left,
				BMP_MAGNETO_DOWN,
				NULL,
				fail,
				IMAGE_USE_ERASE,
				0,
				{29, 45},
				ENGINE1_MAGNETO_LEFT, NULL,
				ICON_SWITCH_TYPE_SET_CUR_ICON,
				2,
				0,
				0 )


MAKE_ICON(		icon_right,
				BMP_MAGNETO_DOWN,
				NULL,
				fail,
				IMAGE_USE_ERASE,
				0,
				{186, 45},
				ENGINE1_MAGNETO_RIGHT, NULL,
				ICON_SWITCH_TYPE_SET_CUR_ICON,
				2,
				0,
				0 )


PELEMENT_HEADER		plist[] =
{
	&icon_left.header,
	&icon_right.header,
	NULL
};

MAKE_STATIC(	bg,
				BMP_BACKGROUND,
				&plist,
				NULL,
				IMAGE_USE_TRANSPARENCY,
				0,
				{0,0} )


PELEMENT_HEADER		list	= &bg.header;


MOUSE_FUNCTION		left_mag_cb;
MOUSE_FUNCTION		right_mag_cb;

MOUSE_BEGIN( mouse_rect, HELPID_SOPWITH_MAG, 0, 0 )
	MOUSE_CHILD_FUNCT( 0, 0, 116, 113, CURSOR_HAND, MOUSE_LEFTSINGLE, left_mag_cb )
	MOUSE_CHILD_FUNCT( 157, 0, 116, 113, CURSOR_HAND, MOUSE_LEFTSINGLE, right_mag_cb )
MOUSE_END


//---------------------------------------------------------------------------
BOOL FSAPI	left_mag_cb( PPIXPOINT relative_point, FLAGS32 mouse_flags )
{
	// play a sound buffer exaclty once
	// There are different play macros for file and resource sounds. The
	// reason behind this is, that sound buffers can get lost (e.g. when the
	// application looses input focus) and in such a case, the FSSound module
	// returns false from it's Play() function. To restore a lost sound buffer,
	// one can simply reregister it with the apropriate register function
	// (RegisterFile() or RegisterRes()). This is exactly what the play macros
	// do behind the scene and that's the reason for the two different macros.
	MFSSoundFilePlay(sound1, FALSE);
	lookup_var( &starter );

	if( starter.var_value.e == ENG_STARTER_MAGNETO_OFF )
	{
		// jump to ENG_STARTER_MAGNETO_LEFT
		trigger_key_event( KEY_MAGNETO_LEFT, 0 );
	}
	else
	if( starter.var_value.e == ENG_STARTER_MAGNETO_LEFT )
	{
		// jump to ENG_STARTER_MAGNETO_OFF
		trigger_key_event( KEY_MAGNETO_OFF, 0 );
	}
	else
	if( starter.var_value.e == ENG_STARTER_MAGNETO_RIGHT )
	{
		// jump to ENG_STARTER_MAGNETO_BOTH
		trigger_key_event( KEY_MAGNETO_BOTH, 0 );
	}
	else
	if( starter.var_value.e == ENG_STARTER_MAGNETO_BOTH )
	{
		// jump to ENG_STARTER_MAGNETO_RIGHT
		trigger_key_event( KEY_MAGNETO_RIGHT, 0 );
	}

	return TRUE;
}

BOOL FSAPI	right_mag_cb( PPIXPOINT relative_point, FLAGS32 mouse_flags )
{
	// start and stop a looping sound buffer
	if (MFSSoundIsPlaying(sound2))
		MFSSoundStop(sound2);
	else
		MFSSoundResPlay(sound2, TRUE);

	lookup_var( &starter );

	if( starter.var_value.e == ENG_STARTER_MAGNETO_OFF )
	{
		// jump to ENG_STARTER_MAGNETO_RIGHT
		trigger_key_event( KEY_MAGNETO_RIGHT, 0 );
	}
	else
	if( starter.var_value.e == ENG_STARTER_MAGNETO_LEFT )
	{
		// jump to ENG_STARTER_MAGNETO_BOTH
		trigger_key_event( KEY_MAGNETO_BOTH, 0 );
	}
	else
	if( starter.var_value.e == ENG_STARTER_MAGNETO_RIGHT )
	{
		// jump to ENG_STARTER_MAGNETO_OFF
		trigger_key_event( KEY_MAGNETO_OFF, 0 );
	}
	else
	if( starter.var_value.e == ENG_STARTER_MAGNETO_BOTH )
	{
		// jump to ENG_STARTER_MAGNETO_LEFT
		trigger_key_event( KEY_MAGNETO_LEFT, 0 );
	}

	return TRUE;
}


//---------------------------------------------------------------------------
void FSAPI	install_routine( HINSTANCE resource_file_handle )
{
	// sounds need to be registered before they can be played
	MFSSoundFileRegister(sound1);
	// for sound resources, the handle of the module must be specified
	MFSSoundResRegister(sound2, resource_file_handle);

	element_list_install( list, resource_file_handle );
	initialize_var( &starter );
	mouse_list_install( mouse_rect, &gauge_header, &bg.image_data.final->dim );
	mouse_list_register( mouse_rect, &gauge_header );
}


//---------------------------------------------------------------------------
void FSAPI	initialize_routine()
{
	element_list_initialize( list );
}


//---------------------------------------------------------------------------
void FSAPI	update_routine()
{
	element_list_update(list);
}


//---------------------------------------------------------------------------
void FSAPI	generate_routine( GENERATE_PHASE phase )
{
	element_list_generate( list, phase );
}


//---------------------------------------------------------------------------
void FSAPI	draw_routine()
{
	element_list_plot( list );
}

//---------------------------------------------------------------------------
void FSAPI	kill_routine()
{
	mouse_list_unregister( mouse_rect, &gauge_header );
	element_list_kill( list );

	// never forget to unregister the sounds
	MFSSoundUnregister(sound1);
	MFSSoundUnregister(sound2);
}

