Attribute VB_Name = "math"
Option Explicit

Function IsaSat(dblAlt As Double) As Double
    If dblAlt > 36000 Then
        IsaSat = -56.5
    Else
        IsaSat = 15# - dblAlt / 1000# * 1.98
    End If
End Function

Function TatFromOat(dblOat As Double, dblMach As Double) As Double
    Rem returns total air temperature (TAT)
    Rem input:
    Rem       outside air temperature (OAT) in degrees centigrade
    Rem       mach number
    Rem TAT is higher than OAT due to friction at aircraft body
    TatFromOat = (1 + 0.2 * dblMach * dblMach) * (dblOat + 273.15) - 273.15
End Function


Function OatFromTat(dblTat As Double, dblMach As Double) As Double

    Rem returns outside air temperature (OAT)
    Rem input:
    Rem       total air temperature (TAT) in degrees centigrade
    Rem       mach number
    Rem OAT is lower than TAT due to friction at aircraft body
    OatFromTat = (dblTat + 273.15) / (1 + 0.2 * dblMach * dblMach) - 273.15
End Function


Function PCorrFromAlt(dblAlt As Double) As Double

    Rem returns pressure correction factor for following calculations
    Rem input:
    Rem      altitude in feet
    If dblAlt > 140000 Then
       PCorrFromAlt = 0
    Else
        PCorrFromAlt = (1# - 3.566 * dblAlt / 518670) ^ (-5.2563)
    End If
End Function


Function MachFromIas(dblAlt As Double, dblIas As Double) As Double
    Rem returns mach number
Rem input:
    Rem       altitude in feet
    Rem       indicated airspeed in knots
    Dim PressureCorrection As Double
    PressureCorrection = PCorrFromAlt(dblAlt)
    Dim dblValue As Double
    
    dblValue = 1 + 0.2 * dblIas * dblIas / 437582.25
    If (dblValue < 0) Then
         MachFromIas = 0
    Else
        dblValue = Sqr(dblValue * dblValue * dblValue * dblValue * dblValue * dblValue * dblValue) - 1
        dblValue = dblValue * PressureCorrection
        dblValue = dblValue + 1#
        If (dblValue < 0) Then
            MachFromIas = 0
        Else
            dblValue = 5 * (dblValue ^ (2# / 7#) - 1#)
            If (dblValue < 0) Then
                MachFromIas = 0
            Else
                MachFromIas = Sqr(dblValue)
            End If
        End If
    End If
End Function


Function IasFromMach(dblAlt As Double, dblMach As Double) As Double
    Rem returns indicated airspeed in knots
Rem input:
    Rem       altitude in feet
    Rem       mach number
    Dim PressureCorrection As Double
    Dim dblValue As Double
    PressureCorrection = PCorrFromAlt(dblAlt)
    
    dblValue = 1# + 0.2 * dblMach * dblMach
    dblValue = Sqr(dblValue * dblValue * dblValue * dblValue * dblValue * dblValue * dblValue) - 1
    dblValue = dblValue / PressureCorrection
    dblValue = dblValue + 1
    If (dblValue < 0) Then
        IasFromMach = 0
    Else
        dblValue = 5 * (dblValue ^ (2# / 7#) - 1)
        If (dblValue < 0) Then
            IasFromMach = 0
        Else
            IasFromMach = Sqr(dblValue) * 661.5
        End If
    End If
End Function

Function TasFromMachTat(dblTatC As Double, dblMach As Double) As Double
    Rem returns true airspeed in knots
Rem input:
    Rem      true air temperature in degrees centigrade
    Rem      mach number
    TasFromMachTat = 39# * dblMach * Sqr((dblTatC + 273.15) / (1 + 0.2 * dblMach * dblMach))
End Function

Function MachFromTasTat(dblTatC As Double, dblTas As Double)
    Rem returns mach number
Rem input:
    Rem      true air temperature in degrees centigrade
    Rem      true airspeed in knots
    Dim dblValue As Double
    dblValue = 39# * 39# / dblTas / dblTas * (dblTatC + 273.15) - 0.2
    If (dblValue <= 0) Then
         MachFromTasTat = 0
    Else
         MachFromTasTat = 1# / Sqr(dblValue)
    End If
End Function

Function TasFromMachOat(dblOatC As Double, dblMach As Double) As Double
    Rem returns true airspeed in knots
Rem input:
    Rem      outside air temperature in degrees centigrade
    Rem      mach number
    TasFromMachOat = 39# * dblMach * Sqr(dblOatC + 273.15)
End Function

 Function MachFromTasOat(dblOatC As Double, dblTas As Double) As Double
    Rem returns mach number
Rem input:
    Rem      outside air temperature in degrees centigrade
    Rem      true airspeed in knots
    Dim dblValue As Double
    dblValue = dblOatC + 273.15
    If (dblValue <= 0) Then
        MachFromTasOat = 0
    Else
        MachFromTasOat = dblTas / 39 / Sqr(dblValue)
    End If
 End Function


