// --------------------------------------------------------------------------
// GENERAL DEFINITIONS
// --------------------------------------------------------------------------
// mouse callbacks
BOOL FSAPI mouse_left_cb(PPIXPOINT, FLAGS32);
BOOL FSAPI mouse_right_cb(PPIXPOINT, FLAGS32);


// callbacks for the icons
FLOAT64 FSAPI left_cb(PELEMENT_ICON);
FLOAT64 FSAPI right_cb(PELEMENT_ICON);


// main gauge callback
void FSAPI over_cb(PGAUGEHDR, int, UINT32);


// local variables
char over_gauge_name[] = GAUGE_NAME;
extern PELEMENT_HEADER over_list;
extern MOUSERECT over_mouse_rect[];


// variables/definitions for the sound buffers:
// set up the vars for two sound buffers
// the first one uses a sound file that is compiled into the gauge
MFSSoundResVars(left_sound, SOUND_OUTER);
// the second one uses a sound file in the main FS sound folder
MFSSoundFileVars(right_sound, "SOUND\\OUTERMK.WAV");



// --------------------------------------------------------------------------
// MODULE IMPORT AND EXPORT DEFINITIONS
// --------------------------------------------------------------------------
GAUGE_HEADER_FS700(GAUGE_W, over_gauge_name, &over_list, \
	over_mouse_rect, over_cb, 0, 0, 0);



// --------------------------------------------------------------------------
// DRAWING ELEMENTS DEFINITIONS
// --------------------------------------------------------------------------
MAKE_ICON(
	left_button,
	BMP_OFF,
	NULL,
	NULL,
	IMAGE_USE_BRIGHT | IMAGE_USE_TRANSPARENCY,
	0,
	0, 0,
	MODULE_VAR_NONE, left_cb,
	ICON_SWITCH_TYPE_SET_CUR_ICON,
	2,
	0,
	0)

MAKE_ICON(
	right_button,
	BMP_OFF,
	NULL,
	NULL,
	IMAGE_USE_BRIGHT | IMAGE_USE_TRANSPARENCY,
	0,
	50, 0,
	MODULE_VAR_NONE, right_cb,
	ICON_SWITCH_TYPE_SET_CUR_ICON,
	2,
	0,
	0)


// link the drawing elements to the background
PELEMENT_HEADER	plist1[] = {
	&left_button.header,
	&right_button.header,
	NULL};

MAKE_STATIC(
	over_background,
	BMP_BACKGROUND,
	&plist1,
	NULL,
	0,
	0,
	0,0)

PELEMENT_HEADER over_list = &over_background.header;



// --------------------------------------------------------------------------
// MOUSE SPECIFIC DEFINITIONS
// --------------------------------------------------------------------------
MOUSE_BEGIN(over_mouse_rect, HELP_NONE, 0, 0)
	MOUSE_CHILD_FUNCT(0, 0, 50, 36, CURSOR_HAND, MOUSE_LEFTSINGLE, mouse_left_cb)
	MOUSE_CHILD_FUNCT(50, 0, 50, 36, CURSOR_HAND, MOUSE_LEFTSINGLE, mouse_right_cb)
MOUSE_END



// --------------------------------------------------------------------------
// MOUSE CALLBACKS
// --------------------------------------------------------------------------
BOOL FSAPI mouse_left_cb(PPIXPOINT relative_point, FLAGS32 mouse_flags) {
	// play a sound buffer exaclty once
	// There are different play macros for file and resource sounds. The
	// reason behind this is, that sound buffers can get lost (e.g. when the
	// application looses input focus) and in such a case, the FSSound module
	// returns false from it's Play() function. To restore a lost sound buffer,
	// one can simply reregister it with the apropriate register function
	// (RegisterFile() or RegisterRes()). This is exactly what the play macros
	// do behind the scene and that's the reason for the two different macros.
	MFSSoundResPlay(left_sound, FALSE);
	return FALSE;
}

BOOL FSAPI mouse_right_cb(PPIXPOINT relative_point, FLAGS32 mouse_flags) {
	// play a sound buffer from a sound file
	MFSSoundFilePlay(right_sound, FALSE);
	return FALSE;
}



// --------------------------------------------------------------------------
// DRAWING ELEMENT CALLBACKS
// --------------------------------------------------------------------------
// icon callbacks
FLOAT64 FSAPI left_cb(PELEMENT_ICON pelement) {
	// you can check whether a sound buffer is currently playing
	return MFSSoundIsPlaying(left_sound);
}

FLOAT64 FSAPI right_cb(PELEMENT_ICON pelement) {
	// the button shows "on" while the sound buffer is playing
	return MFSSoundIsPlaying(right_sound);
}



// --------------------------------------------------------------------------
// MAIN GAUGE CALLBACK
// --------------------------------------------------------------------------
void FSAPI over_cb(PGAUGEHDR pgauge, int service_id, UINT32 extra_data) {
	switch(service_id) {
	case PANEL_SERVICE_POST_INSTALL:
		// sounds need to be registered before they can be played
		MFSSoundFileRegister(right_sound);
		// for sound resources, the handle of the module must be specified
		MFSSoundResRegister(left_sound, (HMODULE)extra_data);

		// the sound controlled by the left button should only be played
		// on the left seaker
		MFSSoundSetPan(left_sound, -10000);
		// same for the sound controlled by the right button
		MFSSoundSetPan(right_sound, 10000);
		break;

	case PANEL_SERVICE_PRE_KILL:
		// never forget to unregister the sounds
		MFSSoundUnregister(left_sound);
		MFSSoundUnregister(right_sound);
		break;
	}
}



// --------------------------------------------------------------------------
// CLEAN UP
// --------------------------------------------------------------------------
#undef GAUGE_NAME
#undef GAUGEHDR_VAR_NAME
#undef GAUGE_W

