// --------------------------------------------------------------------------
// GENERAL DEFINITIONS
// --------------------------------------------------------------------------
// mouse callbacks
BOOL FSAPI mouse_button_cb(PPIXPOINT, FLAGS32);
BOOL FSAPI mouse_pan_cb(PPIXPOINT, FLAGS32);
BOOL FSAPI mouse_vol_cb(PPIXPOINT, FLAGS32);
BOOL FSAPI mouse_freq_cb(PPIXPOINT, FLAGS32);


// callbacks for the icons
FLOAT64 FSAPI button_cb(PELEMENT_ICON);


// callbacks for the sliders
FLOAT64 FSAPI pan_cb(PELEMENT_SLIDER);
FLOAT64 FSAPI vol_cb(PELEMENT_SLIDER);
FLOAT64 FSAPI freq_cb(PELEMENT_SLIDER);


// main gauge callback
void FSAPI main_cb(PGAUGEHDR, int, UINT32);


// local variables
char main_gauge_name[] = GAUGE_NAME;
extern PELEMENT_HEADER main_list;
extern MOUSERECT main_mouse_rect[];
// this variable will hold the actual width of the gauge; it is needed in some
// mouse callbacks
double width;


// variables/definitions for the sound buffers:
// a sound buffer used to demonstrate the various effects
MFSSoundFileVars(main_sound, "SOUND\\MIDDMK.WAV");



// --------------------------------------------------------------------------
// MODULE IMPORT AND EXPORT DEFINITIONS
// --------------------------------------------------------------------------
GAUGE_HEADER_FS700(GAUGE_W, main_gauge_name, &main_list, \
						main_mouse_rect, main_cb, 0, 0, 0);


// --------------------------------------------------------------------------
// DRAWING ELEMENTS DEFINITIONS
// --------------------------------------------------------------------------
MAKE_ICON(
	button,
	BMP_OFF,
	NULL,
	NULL,
	IMAGE_USE_BRIGHT | IMAGE_USE_TRANSPARENCY,
	0,
	75, 0,
	MODULE_VAR_NONE, button_cb,
	ICON_SWITCH_TYPE_SET_CUR_ICON,
	2,
	0,
	0)

MAKE_SLIDER(
	pan_slider,
	BMP_SLIDER,
	NULL,
	0,
	IMAGE_USE_ERASE | IMAGE_USE_TRANSPARENCY,
	0,
	90, 46,

	MODULE_VAR_NONE, pan_cb, 0.0075,
	MODULE_VAR_NONE, NULL, 0)

MAKE_SLIDER(
	vol_slider,
	BMP_SLIDER,
	NULL,
	0,
	IMAGE_USE_ERASE | IMAGE_USE_TRANSPARENCY,
	0,
	165, 92,

	MODULE_VAR_NONE, vol_cb, 0.0150,
	MODULE_VAR_NONE, NULL, 0)

MAKE_SLIDER(
	freq_slider,
	BMP_SLIDER,
	NULL,
	0,
	IMAGE_USE_ERASE | IMAGE_USE_TRANSPARENCY,
	0,
	15, 138,

	MODULE_VAR_NONE, freq_cb, 0.00375,
	MODULE_VAR_NONE, NULL, 0)


// link the drawing elements to the background
PELEMENT_HEADER	plist2[] = {
	&button.header,
	&pan_slider.header,
	&vol_slider.header,
	&freq_slider.header,
	NULL};

MAKE_STATIC(
	main_background,
	BMP_BACKGROUND2,
	&plist2,
	NULL,
	0,
	0,
	0,0)

PELEMENT_HEADER main_list = &main_background.header;



// --------------------------------------------------------------------------
// MOUSE SPECIFIC DEFINITIONS
// --------------------------------------------------------------------------
MOUSE_BEGIN(main_mouse_rect, HELP_NONE, 0, 0)
	MOUSE_CHILD_FUNCT(75, 0, 50, 36, CURSOR_HAND, MOUSE_LEFTSINGLE, mouse_button_cb)
	MOUSE_CHILD_FUNCT(25, 46, 150, 36, CURSOR_HAND, MOUSE_LEFTSINGLE|MOUSE_LEFTDRAG, mouse_pan_cb)
	MOUSE_CHILD_FUNCT(25, 92, 150, 36, CURSOR_HAND, MOUSE_LEFTSINGLE|MOUSE_LEFTDRAG, mouse_vol_cb)
	MOUSE_CHILD_FUNCT(25, 138, 150, 36, CURSOR_HAND, MOUSE_LEFTSINGLE|MOUSE_LEFTDRAG, mouse_freq_cb)
MOUSE_END



// --------------------------------------------------------------------------
// MOUSE CALLBACKS
// --------------------------------------------------------------------------
BOOL FSAPI mouse_button_cb(PPIXPOINT relative_point, FLAGS32 mouse_flags) {
	// start or stop a looping sound buffer
	if (MFSSoundIsPlaying(main_sound))
		MFSSoundStop(main_sound);
	else
		// specifying TRUE as the second parameter in the play function (macro)
		// makes the sound buffer loop forever or until the stop function
		// (macro) is called
		MFSSoundFilePlay(main_sound, TRUE);

	return	FALSE;
}

BOOL FSAPI mouse_pan_cb(PPIXPOINT relative_point, FLAGS32 mouse_flags) {
	// set the pan of the soundbuffer according to the position of the
	// slider
	MFSSoundSetPan(main_sound, ((double)relative_point->x / width)
		* (double)GAUGE_W * 133.3333333333 - 10000.0);
	return	FALSE;
}

BOOL FSAPI mouse_vol_cb(PPIXPOINT relative_point, FLAGS32 mouse_flags) {
	// set the volume of the sound buffer according to the position of the
	// slider
	MFSSoundSetVolume(main_sound, ((double)relative_point->x / width)
		* (double)GAUGE_W * 66.66666666667 - 10000.0);
	return	FALSE;
}

BOOL FSAPI mouse_freq_cb(PPIXPOINT relative_point, FLAGS32 mouse_flags) {
	double val = ((double)relative_point->x / width) * (double)GAUGE_W
		* 266.6666666666;

	// assert: 100Hz >= frequency <= 40000Hz
	// (the highest frequency possible would be 100000Hz)
	if (val > 40000) val = 40000;
	else if (val < 100) val = 100;

	// set the frequency of the sound buffer according to the position of the
	// slider
	MFSSoundSetFrequency(main_sound, val);
	return	FALSE;
}



// --------------------------------------------------------------------------
// DRAWING ELEMENT CALLBACKS
// --------------------------------------------------------------------------
// icon callbacks
FLOAT64 FSAPI button_cb(PELEMENT_ICON pelement) {
	// control the state of the button according to whether the sound buffer
	// is currently played back
	return MFSSoundIsPlaying(main_sound);
}


// slider callbacks
FLOAT64 FSAPI pan_cb(PELEMENT_SLIDER pelement) {
	// get the actual pan of a sound buffer
	return MFSSoundGetPan(main_sound);
}

FLOAT64 FSAPI vol_cb(PELEMENT_SLIDER pelement) {
	// get the actual volume of a sound buffer
	return MFSSoundGetVolume(main_sound);
}

FLOAT64 FSAPI freq_cb(PELEMENT_SLIDER pelement) {
	// get the actual frequency used for playing back a sound buffer
	return MFSSoundGetFrequency(main_sound);
}



// --------------------------------------------------------------------------
// MAIN GAUGE CALLBACK
// --------------------------------------------------------------------------
void FSAPI main_cb(PGAUGEHDR pgauge, int service_id, UINT32 extra_data) {
	switch(service_id) {
	case PANEL_SERVICE_POST_INSTALL:
		// a sound buffer must be registered before it can be played back
		MFSSoundFileRegister(main_sound);
		break;

	case PANEL_SERVICE_POST_INITIALIZE:
		// save the actual with of the gauge
		width = ((PELEMENT_STATIC_IMAGE)pgauge->elements_list[0])
			->image_data.final->dim.x;
		break;

	case PANEL_SERVICE_PRE_KILL:
		// never forget to unregister the sound buffers
		MFSSoundUnregister(main_sound);
		break;
	}
}



// --------------------------------------------------------------------------
// CLEAN UP
// --------------------------------------------------------------------------
#undef GAUGE_NAME
#undef GAUGEHDR_VAR_NAME
#undef GAUGE_W

