// include the FSSound header file
#include "FSSound.h"

#include "gauges.h"
#include "fs98.sound.h"



// --------------------------------------------------------------------------
// GENERAL DEFINITIONS
// --------------------------------------------------------------------------
// mouse callbacks
MOUSE_FUNCTION mouse_left_cb;
MOUSE_FUNCTION mouse_right_cb;
MOUSE_FUNCTION mouse_button_cb;
MOUSE_FUNCTION mouse_pan_cb;
MOUSE_FUNCTION mouse_vol_cb;
MOUSE_FUNCTION mouse_freq_cb;


// callbacks for the icons
MODULE_VAR_CB left_cb;
MODULE_VAR_CB right_cb;
MODULE_VAR_CB master_cb;
MODULE_VAR_CB button_cb;


// callbacks for the sliders
MODULE_VAR_CB pan_cb;
MODULE_VAR_CB vol_cb;
MODULE_VAR_CB freq_cb;


// local variables/macros
char gauge_name[] = "fs98.sound\0";
extern PELEMENT_HEADER list;
#define GAUGE_W 200

// variables/definitions for the sound buffers:
// the first one uses a sound file that is compiled into the gauge as a
// resource
MFSSoundResVars(left_sound, SOUND_OUTER);
// the second one uses a sound file in the main FS sound folder
MFSSoundFileVars(right_sound, "SOUND\\OUTERMK.WAV");
// the third one is also read from a file and is used to test the varous
// effects
MFSSoundFileVars(main_sound, "SOUND\\MIDDMK.WAV");



// --------------------------------------------------------------------------
// MODULE IMPORT AND EXPORT DEFINITIONS
// --------------------------------------------------------------------------
MODULE_SETUP(GAUGE_HEADER_VERSION, GAUGE_W, gauge_name, &list);



// --------------------------------------------------------------------------
// DRAWING ELEMENTS DEFINITIONS
// --------------------------------------------------------------------------
MAKE_ICON(
	left_button,
	BMP_OFF,
	NULL,
	NULL,
	IMAGE_USE_BRIGHT | IMAGE_USE_TRANSPARENCY,
	0,
	{15, 0},
	MODULE_VAR_NONE, left_cb,
	ICON_SWITCH_TYPE_SET_CUR_ICON,
	2,
	0,
	0)

MAKE_ICON(
	right_button,
	BMP_OFF,
	NULL,
	NULL,
	IMAGE_USE_BRIGHT | IMAGE_USE_TRANSPARENCY,
	0,
	{135, 0},
	MODULE_VAR_NONE, right_cb,
	ICON_SWITCH_TYPE_SET_CUR_ICON,
	2,
	0,
	0)

MAKE_ICON(
	main_button,
	BMP_OFF,
	NULL,
	NULL,
	IMAGE_USE_BRIGHT | IMAGE_USE_TRANSPARENCY,
	0,
	{75, 0},
	MODULE_VAR_NONE, button_cb,
	ICON_SWITCH_TYPE_SET_CUR_ICON,
	2,
	0,
	0)

MAKE_SLIDER(
	pan_slider,
	BMP_SLIDER,
	NULL,
	0,
	IMAGE_USE_ERASE | IMAGE_USE_TRANSPARENCY,
	0,
	{90, 46},

	MODULE_VAR_NONE, pan_cb, 0.0075,
	MODULE_VAR_NONE, NULL, 0)

MAKE_SLIDER(
	vol_slider,
	BMP_SLIDER,
	NULL,
	0,
	IMAGE_USE_ERASE | IMAGE_USE_TRANSPARENCY,
	0,
	{165, 92},

	MODULE_VAR_NONE, vol_cb, 0.0150,
	MODULE_VAR_NONE, NULL, 0)

MAKE_SLIDER(
	freq_slider,
	BMP_SLIDER,
	NULL,
	0,
	IMAGE_USE_ERASE | IMAGE_USE_TRANSPARENCY,
	0,
	{15, 138},

	MODULE_VAR_NONE, freq_cb, 0.00375,
	MODULE_VAR_NONE, NULL, 0)


// link the drawing elements to the background
PELEMENT_HEADER	plist[] = {
	&left_button.header,
	&right_button.header,
	&main_button.header,
	&pan_slider.header,
	&vol_slider.header,
	&freq_slider.header,
	NULL};

MAKE_STATIC(
	bg,
	BMP_BACKGROUND,
	&plist,
	NULL,
	0,
	0,
	{0,0})

PELEMENT_HEADER list = &bg.header;



// --------------------------------------------------------------------------
// MOUSE SPECIFIC DEFINITIONS
// --------------------------------------------------------------------------
MOUSERECT mouse_rect[] = {
MOUSE_PARENT_BEGIN(0, 0, 0, 0, HELP_NONE)
	MOUSE_CHILD_FUNCT(15, 0, 50, 36, CURSOR_HAND, MOUSE_LEFTSINGLE, mouse_left_cb)
	MOUSE_CHILD_FUNCT(135, 0, 50, 36, CURSOR_HAND, MOUSE_LEFTSINGLE, mouse_right_cb)
	MOUSE_CHILD_FUNCT(75, 0, 50, 36, CURSOR_HAND, MOUSE_LEFTSINGLE, mouse_button_cb)
	MOUSE_CHILD_FUNCT(25, 46, 150, 36, CURSOR_HAND, MOUSE_LEFTSINGLE|MOUSE_LEFTDRAG, mouse_pan_cb)
	MOUSE_CHILD_FUNCT(25, 92, 150, 36, CURSOR_HAND, MOUSE_LEFTSINGLE|MOUSE_LEFTDRAG, mouse_vol_cb)
	MOUSE_CHILD_FUNCT(25, 138, 150, 36, CURSOR_HAND, MOUSE_LEFTSINGLE|MOUSE_LEFTDRAG, mouse_freq_cb)
	MOUSE_PARENT_END
	{MOUSE_RECT_EOL, {0, 0, 0, 0 }, CURSOR_NONE, HELP_NONE, MOUSE_NONE, 0, NULL, NULL}};



// --------------------------------------------------------------------------
// MOUSE CALLBACKS
// --------------------------------------------------------------------------
BOOL FSAPI mouse_left_cb(PPIXPOINT relative_point, FLAGS32 mouse_flags) {
	// play a sound buffer exaclty once
	// There are different play macros for file and resource sounds. The
	// reason behind this is, that sound buffers can get lost (e.g. when the
	// application looses input focus) and in such a case, the FSSound module
	// returns false from it's Play() function. To restore a lost sound buffer,
	// one can simply reregister it with the apropriate register function
	// (RegisterFile() or RegisterRes()). This is exactly what the play macros
	// do behind the scene and that's the reason for the two different macros.
	MFSSoundResPlay(left_sound, FALSE);
	return FALSE;
}

BOOL FSAPI mouse_right_cb(PPIXPOINT relative_point, FLAGS32 mouse_flags) {
	// play a sound buffer from a sound file
	MFSSoundFilePlay(right_sound, FALSE);
	return FALSE;
}

BOOL FSAPI mouse_button_cb(PPIXPOINT relative_point, FLAGS32 mouse_flags) {
	// start or stop a looping sound buffer
	if (MFSSoundIsPlaying(main_sound))
		MFSSoundStop(main_sound);
	else
		// specifying TRUE as the second parameter in the play function (macro)
		// makes the sound buffer loop forever or until the stop function
		// (macro) is called
		MFSSoundFilePlay(main_sound, TRUE);

	return	FALSE;
}

BOOL FSAPI mouse_pan_cb(PPIXPOINT relative_point, FLAGS32 mouse_flags) {
	// set the pan of the soundbuffer according to the position of the
	// slider
	MFSSoundSetPan(main_sound, ((double)relative_point->x
		/ (double)(bg.image_data.final->dim.x))
		* (double)GAUGE_W * 133.3333333333 - 10000.0);
	return	FALSE;
}

BOOL FSAPI mouse_vol_cb(PPIXPOINT relative_point, FLAGS32 mouse_flags) {
	// set the volume of the sound buffer according to the position of the
	// slider
	MFSSoundSetVolume(main_sound, ((double)relative_point->x
		/ (double)(bg.image_data.final->dim.x))
		* (double)GAUGE_W * 66.66666666667 - 10000.0);
	return	FALSE;
}

BOOL FSAPI mouse_freq_cb(PPIXPOINT relative_point, FLAGS32 mouse_flags) {
	double val = ((double)relative_point->x
		/ (double)(bg.image_data.final->dim.x))
		* (double)GAUGE_W * 266.6666666666;

	// assert: 100Hz >= frequency <= 40000Hz
	// (the highest frequency possible would be 100000Hz)
	if (val > 40000) val = 40000;
	else if (val < 100) val = 100;

	// set the frequency of the sound buffer according to the position of the
	// slider
	MFSSoundSetFrequency(main_sound, val);
	return	FALSE;
}



// --------------------------------------------------------------------------
// DRAWING ELEMENT CALLBACKS
// --------------------------------------------------------------------------
// icon callbacks
FLOAT64 FSAPI left_cb(PMODULE_VAR var) {
	// you can check whether a sound buffer is currently playing
	return MFSSoundIsPlaying(left_sound);
}

FLOAT64 FSAPI right_cb(PMODULE_VAR var) {
	// the button shows "on" while the sound buffer is playing
	return MFSSoundIsPlaying(right_sound);
}

FLOAT64 FSAPI button_cb(PMODULE_VAR var) {
	// control the state of the button according to whether the looping sound
	// buffer is currently playing
	return MFSSoundIsPlaying(main_sound);
}


// slider callbacks
FLOAT64 FSAPI pan_cb(PMODULE_VAR var) {
	// get the actual pan of a sound buffer
	return MFSSoundGetPan(main_sound);
}

FLOAT64 FSAPI vol_cb(PMODULE_VAR var) {
	// get the actual volume of a sound buffer
	return MFSSoundGetVolume(main_sound);
}

FLOAT64 FSAPI freq_cb(PMODULE_VAR var) {
	// get the actual frequency used for playing back a sound buffer
	return MFSSoundGetFrequency(main_sound);
}



// --------------------------------------------------------------------------
// INTERFACE FUNCTIONS
// --------------------------------------------------------------------------
void FSAPI install_routine(HINSTANCE resource_file_handle) {
	// a sound buffer must be registered before it can be played back
	MFSSoundFileRegister(main_sound);
	MFSSoundFileRegister(right_sound);
	// for sound resources, the handle of the module must be specified
	MFSSoundResRegister(left_sound, resource_file_handle);

	// the sound controlled by the left button should only be played
	// on the left seaker
	MFSSoundSetPan(left_sound, -10000);
	// same for the sound controlled by the right button
	MFSSoundSetPan(right_sound, 10000);

	element_list_install(list, resource_file_handle);
	mouse_list_install(mouse_rect, &gauge_header, &bg.image_data.final->dim);
	mouse_list_register(mouse_rect, &gauge_header);
}

void FSAPI	initialize_routine() {
	element_list_initialize( list );
}

void FSAPI update_routine() {
	element_list_update(list);
}

void FSAPI generate_routine(GENERATE_PHASE phase) {
	element_list_generate(list, phase);
}

void FSAPI draw_routine() {
	element_list_plot(list);
}

void FSAPI kill_routine() {
	mouse_list_unregister(mouse_rect, &gauge_header);
	element_list_kill(list);

	// never forget to unregister the sounds
	MFSSoundUnregister(main_sound);
	MFSSoundUnregister(left_sound);
	MFSSoundUnregister(right_sound);
}
